# include <genanneal.h>
# include <math.h>
# include <get_options_anneal.h>

GenAnneal::GenAnneal(Problem *p)
{
	problem = p;
	int dimension=problem->getDimension();
	x.resize(dimension);
	problem->getSample(x);
	rt=0.5;
	eps=1e-5;
	ns=20;
	nt=5;
	neps=4;
	c.resize(dimension);
	for(int i=0;i<dimension;i++) c[i]=2.0;
	t=5.0;
	xopt.resize(dimension);
	Solver=new Grs(problem);
	Solver->setGenomeCount(genome_count);
	Solver->setGenomeLength(10 * problem->getDimension());
}

void    GenAnneal::setRT(double r)
{
	rt = r;
}

double  GenAnneal::getRT() const
{
	return rt;
}

void    GenAnneal::setEPS(double e)
{
	eps = e;
}

double  GenAnneal::getEPS() const
{
	return eps;
}

void    GenAnneal::setT(double T)
{
	t = T;
}

double  GenAnneal::getT() const
{
	return t;
}

void    GenAnneal::setNEPS(int n)
{
	neps = n;
}

int     GenAnneal::getNEPS() const
{
	return neps;
}

double  GenAnneal::getMinimum() 
{
	return fopt;
}

double  GenAnneal::getMinimum(Data &xx) 
{
	xx = xopt;
	return fopt;
}

void	GenAnneal::Solve()
{
	double f,fp,p,pp,ratio;
	int	h,i,j,m;
	int	quit=0;
	int dimension=problem->getDimension();
	xopt=x;
	Data fstar;
	fstar.resize(neps);
	for(int i=0;i<neps;i++) fstar[i]=1e+20;
	f=problem->funmin(x);
	fopt=f;
	fstar[0]=f;
	Data xp;
	xp.resize(dimension);
	xp=x;
        double y=f;
	xp=x;
	fp=f;
label100:
	Solver->Solve(xp,fp);
	if(fp<=f)
	{
		//accept
		x=xp;
		f=fp;
		if(fp<=fopt)
		{
		//new minimum
			xopt=xp;
			fopt=fp;
		}
	}
	else
	{
		//check metropolis criterion
		p=exp((fp-f)/t);
		pp=drand48();
		if(pp<p)
		{
			 x=xp;
			 f=fp;
		}
	}
label101:
	quit=0;
	fstar[0]=f;
	if((fopt-fstar[0]<=eps)) quit=1;
	for(int i=0;i<neps;i++)
	{
	 if(fabs(f-fstar[i])>eps) 
	 {
	 	quit=0;
	 }
	}
	if(!quit) quit=fabs(t)<eps;
	if(quit)
	{
		x=xopt;
		MinInfo Info;
		Info.p=problem;
		fopt=tolmin(x,Info);
		xopt=x;
		return;
	}
	t=rt * t;
	for(int i=neps;i>=2;i--)
		fstar[i-1]=fstar[i-2];
	goto label100;
}

GenAnneal::~GenAnneal()
{
	delete Solver;
}
